unit gw4ntMain;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls,
  { JEDI JVCL }
  JvJclUtils, ExtCtrls;

const
  WM_STARTUP = WM_USER + 101;  // 1024 + 101

type
  TfrmIBGW4NT7 = class(TForm)
    pnlMain: TPanel;
    Status: TMemo;
    procedure FormShow(Sender: TObject);
  protected
    procedure _WM_Startup(var Msg: TMessage); message WM_STARTUP;
  private
    { Private-Deklarationen }
    hIBGW: HWND;
    procedure Log(const S: string);
    procedure SearchIBGateway;
    function PreparedForNinjaTrader: boolean;
  public
    { Public-Deklarationen }
  end;

var
  frmIBGW4NT7: TfrmIBGW4NT7;

implementation

{$R *.dfm}

{------------------------------------------------------------------------------}

const
  TWSWinTitle  = 'IB Trader Workstation';
  IBGWWinTitle = 'IB Gateway';

{------------------------------------------------------------------------------}

procedure TfrmIBGW4NT7._WM_Startup(var Msg: TMessage);
begin

  Status.Clear;

  Log(Caption);
  Log('');
  Sleep(500);

  Log('Searching IB Gateway...');
  Sleep(500);
  SearchIBGateway;

  if hIBGW = 0 then begin
    Log('ERROR: IB Gateway not found or already prepared!');
  end else begin

    Log('IB Gateway found.');

    Log('Preparing IB Gateway for Ninja Trader...');
    Sleep(500);

    if not SetWindowText(hIBGW, TWSWinTitle) then begin
      Log('ERROR: Cound not prepare IB Gateway for Ninja Trader');
    end else begin

      Log('Verifying...');
      Sleep(500);

      if not PreparedForNinjaTrader then begin
        Log('ERROR: Verify failed!');
        Log('ERROR: IB Gateway not ready for Ninja Trader');
      end else begin
        Log('IB Gateway ready for connection from Ninja Trader');
      end;
    end;
  end;

  Log('');
  Log('The program will be closed automatically in 5 seconds...');
  Sleep(5000);

  { Return IB Gateway window handle as ExitCode for further use }
  ExitCode := hIBGW;

  Close;
  
end;

{------------------------------------------------------------------------------}

procedure TfrmIBGW4NT7.FormShow(Sender: TObject);
begin
  PostMessage(Handle,WM_STARTUP,0,0);
end;

{------------------------------------------------------------------------------}

procedure TfrmIBGW4NT7.Log(const S: string);
begin
  Status.Lines.Add(S);
end;

{------------------------------------------------------------------------------}

procedure TfrmIBGW4NT7.SearchIBGateway;
var
  SL: TStringlist;
  i: integer;
  Wnd: HWND;
begin

  // Search for IB Gateway main window
  //
  // Win-Title: "IB Gateway"
  // Classname: "SunAwtFrame" (last check on 914.8)
  //
  // Enhanced version:
  // - check for JAVA.EXE or JAVAW.EXE as process name
  // - check for "ibgateway.GWClient" main classname
  // The enhanced version could detect a prepared or non-prepared IB Gateway
  //
  // Optional commandline parameters for enhanced version:
  //   /u -> unprepare (sets IB Gateway to default state)

  hIBGW := 0;
  SL := TStringlist.Create;
  try
    GetVisibleWindows(SL);
    for i := 0 to SL.Count-1 do begin
      Wnd := Integer(SL.Objects[i]);
      if (Wnd <> Self.Handle) and // Check for own window!
         (Pos(IBGWWinTitle, SL[i]) > 0) then begin
        hIBGW := Wnd;
        Break;
      end;
    end;
  finally
    SL.Free;
  end;
end;

{------------------------------------------------------------------------------}

function TfrmIBGW4NT7.PreparedForNinjaTrader: boolean;
var
  S1: array [0..256] of Char;
  S2: string;
begin
  Result := False;
  if Windows.IsWindowVisible(hIBGW) then begin
    GetWindowText(hIBGW, S1, SizeOf(S1));
    S2 := S1;
    Result := SameText(S2,TWSWinTitle);
  end;
end;

{------------------------------------------------------------------------------}

end.
